@@ -32,15 +32,31 @@
 #include <richedit.h>
 #undef _RICHEDIT_VER
 
+#define DpiAwarePerMonitor2() ( FALSE ) // Not yet
+#define DpiAwarePerMonitor() ( FALSE )
+#define SupportsWNT4() ( sizeof(void*) == 4 && !DpiAwarePerMonitor() ) // NT4 does not support the MultiMon API
+#define SupportsW9X() ( sizeof(TCHAR) == 1 )
+#define SupportsW95() ( FALSE && SupportsW9X() && !DpiAwarePerMonitor() )
+#define SupportsW2000() ( sizeof(void*) == 4 )
+
+static inline bool IsWin9598ME() { return SupportsW9X() && (int) GetVersion() < 0; }
+static inline bool IsWin95() { return SupportsW95() && (GetVersion() & (0x8000FFFF & ~0x0300)) == 0x80000004; }
+
 // Defines
-#define NSIS_URL     "http://nsis.sourceforge.net/"
+#define NSIS_URL     "https://nsis.sourceforge.io/"
 #define NSIS_FORUM_URL "http://forums.winamp.com/forumdisplay.php?forumid=65"
-#define NSIS_UC_URL  "http://nsis.sourceforge.net/update.php?version="
-#define NSIS_DL_URL  "http://nsis.sourceforge.net/download/"
-#define USAGE        _T("Usage:\r\n\r\n - File | Load Script...\r\n - Drag the .nsi file into this window\r\n - Right click the .nsi file and choose \"Compile NSIS Script\"")
+#define NSIS_UC_URL  "https://nsis.sourceforge.io/update.php?version="
+#define NSIS_DL_URL  "https://nsis.sourceforge.io/download/"
+#ifdef UNICODE
+#define USAGE        _T("Usage:\r\n\r\n \x2022 File \x203a Load Script...\r\n \x2022 Drag the .nsi file into this window\r\n \x2022 Right click the .nsi file and choose \"Compile NSIS Script\"\r\n")
+#else
+#define USAGE        _T("Usage:\r\n\r\n - File | Load Script...\r\n - Drag the .nsi file into this window\r\n - Right click the .nsi file and choose \"Compile NSIS Script\"\r\n")
+#endif
 #define COPYRIGHT    _T("Copyright (C) 2002 Robert Rainwater")
 #define CONTRIB      _T("Fritz Elfert, Justin Frankel, Amir Szekely, Sunil Kamath, Joost Verburg, Anders Kjersem")
-#define DOCPATH      "http://nsis.sourceforge.net/Docs/"
+#define DONATE       _T("<A>Donate</A> to support NSIS development.")
+#define DONATEURL    { 'h'|x,'t','t'|x,'p',':'|x,'/'|x,'/'|x,'n'|x,'s'|x,'i'|x,'s'|x,'.'|x,'s'|x,'f'|x,'.','n'|x,'e'|x,'t'|x,'/'|x,'r','/'|x,'D'|x,'o'|x,'n'|x,'a'|x,'t'|x,'e'|x,'\0' } // "Encrypted"
+#define DOCPATH      "https://nsis.sourceforge.io/Docs/"
 #define LOCALDOCS    _T("\\NSIS.chm")
 #define ERRBOXTITLE  0 //_T("Error")
 #define NSISERROR    _T("Unable to intialize MakeNSIS. Please verify that makensis.exe is in the same directory as makensisw.exe.")
@@ -54,13 +70,14 @@
 #define REGLOC       _T("MakeNSISWPlacement")
 #define REGVERBOSITY _T("MakeNSISWVerbosity")
 #define REGCOMPRESSOR _T("MakeNSISWCompressor")
+#define REGCOLORIZE  _T("MakeNSISWColorize")
 #define REGSYMSUBKEY _T("Symbols")
 #define REGMRUSUBKEY _T("MRU")
 #define EXENAME      _T("makensis.exe")
 #define MAX_STRING   256
 #define TIMEOUT      100
-#define MINWIDTH     350
-#define MINHEIGHT    180
+#define MINWIDTH     400
+#define MINHEIGHT    220
 #define COMPRESSOR_MESSAGE _T("\n\nThe %s compressor created the smallest installer (%d bytes).")
 #define RESTORED_COMPRESSOR_MESSAGE _T("\n\nThe %s compressor created the smallest installer (%d bytes).")
 #define EXE_HEADER_COMPRESSOR_STAT _T("EXE header size:")
@@ -75,23 +92,32 @@
 #define WM_MAKENSIS_PROCESSCOMPLETE (WM_USER+1001)
 #define WM_MAKENSIS_LOADSYMBOLSET (WM_USER+1002)
 #define WM_MAKENSIS_SAVESYMBOLSET (WM_USER+1003)
+#define WM_MAKENSIS_UPDATEUISTATE (WM_USER+1004)
 
 namespace MakensisAPI {
   extern const TCHAR* SigintEventNameFmt;
   extern const TCHAR* SigintEventNameLegacy;
 
-  enum notify_e {
+  enum datatransfer_e {
     NOTIFY_SCRIPT,
     NOTIFY_WARNING,
     NOTIFY_ERROR,
-    NOTIFY_OUTPUT
+    NOTIFY_OUTPUT,
+    PROMPT_FILEPATH
   };
   enum sndmsg_e {
     QUERYHOST = WM_APP
   };
   enum QUERYHOST_e {
-    QH_OUTPUTCHARSET = 1
+    QH_OUTPUTCHARSET = 1,
+    QH_ENABLESTDERR,
+    QH_SUPPORTEDVERSION
   };
+  typedef struct {
+    unsigned char Platform;
+    unsigned char Reserved;
+    TCHAR Path[1];
+  } PROMPT_FILEPATH_DATA;
 }
 
 typedef enum {
@@ -155,13 +181,24 @@ int compressor_strings[] = {IDS_SCRIPT,
 // Extern Variables
 
 extern const TCHAR* NSISW_VERSION;
+extern void* g_ModalDlgData;
 
 DWORD WINAPI MakeNSISProc(LPVOID TreadParam);
 INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
-BOOL CALLBACK DialogResize(HWND hWnd, LPARAM /* unused*/);
-INT_PTR CALLBACK AboutProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
+
+typedef struct {
+  enum { TID_HEADER = 1 };
+  HFONT hHeaderFont, hFont, hBoldFont;
+  INT AnimPos, AnimDir;
+  UINT AnimSpeed, FinalHeaderPos;
+} ABOUTDLGDATA;
+INT_PTR ShowAboutDialog(HWND hwndOwner);
 INT_PTR CALLBACK SettingsProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
-INT_PTR CALLBACK SymbolSetProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
+typedef struct {
+  void*pOldMDD;
+  BOOL LoadingMode;
+} SYMSETDLGDATA;
+INT_PTR ShowSymbolSetDialog(HWND hwndOwner, BOOL LoadingSet);
 INT_PTR CALLBACK CompressorProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam);
 void           SetScript(const TCHAR *script, bool clearArgs = true);
 void           CompileNSISScript();
@@ -186,7 +223,6 @@ typedef struct NSISScriptData {
   char  *brandingv;
   TCHAR **symbols;
   int retcode;
-  bool userSelectCompressor;
   unsigned char verbosity;
   DWORD logLength;
   DWORD warnings;
@@ -200,7 +236,6 @@ typedef struct NSISScriptData {
   HANDLE sigint_event;
   HANDLE sigint_event_legacy;
   HWND focused_hwnd;
-  CHARRANGE textrange;
   NCOMPRESSOR default_compressor;
   NCOMPRESSOR compressor;
   LPCTSTR compressor_name;
@@ -217,6 +252,7 @@ typedef struct ResizeData {
   RECT griprect;
   int dx;
   int dy;
+  UINT bottompanelsize;
 } NRESIZEDATA;
 
 typedef struct FindReplaceDialog {
@@ -231,4 +267,8 @@ typedef struct ToolTipStruct {
   HHOOK hook;
 } NTOOLTIP;
 
+
+extern INT_PTR ShowWndSpy(HWND hOwner);
+extern INT_PTR ShowLookupDialog(HWND hOwner);
+
 #endif
