@@ -60,6 +60,9 @@ HMODULE LoadSysLibrary(LPCSTR Mod)
   return LoadLibrary(path);
 }
 
+#ifdef DECLSPEC_NOINLINE
+DECLSPEC_NOINLINE
+#endif
 FARPROC GetSysProcAddr(LPCSTR Mod, LPCSTR FuncName)
 {
   return GetProcAddress(LoadSysLibrary(Mod), FuncName);
@@ -166,6 +169,25 @@ void SetTitle(HWND hwnd,const TCHAR *substr) {
   SetWindowText(hwnd,title);
 }
 
+typedef struct { LPCSTR SoundName; int MBFallback; } PLAYAPPSOUNDDATA;
+static DWORD CALLBACK PlayAppSoundProc(LPVOID ThreadParam) {
+  PLAYAPPSOUNDDATA *p = (PLAYAPPSOUNDDATA*) ThreadParam;
+  BOOL succ = PlaySoundA(p->SoundName, NULL, (SND_APPLICATION|SND_ALIAS|SND_NODEFAULT) & ~SND_ASYNC); // Cannot use SND_ASYNC because we need to detect if the sound played
+  if (!succ && p->MBFallback >= 0) succ = MessageBeep(p->MBFallback);
+  MemFree(p);
+  return succ;
+}
+
+void PlayAppSoundAsync(LPCSTR SoundName, int MBFallback) {
+  DWORD tid;
+  PLAYAPPSOUNDDATA *p = (PLAYAPPSOUNDDATA*) MemAlloc(sizeof(PLAYAPPSOUNDDATA));
+  if (p) {
+    p->SoundName = SoundName, p->MBFallback = MBFallback; // Note: The string must be valid until the sound has started because we don't copy it
+    HANDLE hThread = CreateThread(NULL, 0, PlayAppSoundProc, p, 0, &tid);
+    if (hThread) CloseHandle(hThread); else PlayAppSoundProc(p);
+  }
+}
+
 void CopyToClipboard(HWND hwnd) {
   if (!hwnd || !OpenClipboard(hwnd)) return;
   LRESULT len = SendDlgItemMessage(hwnd, IDC_LOGWIN, WM_GETTEXTLENGTH, 0, 0);
@@ -185,8 +207,24 @@ void CopyToClipboard(HWND hwnd) {
   CloseClipboard();
 }
 
+void SetLogColor(enum LOGCOLOR lc)
+{
+  enum { em_seteditstyle = (WM_USER + 204), ses_extendbackcolor = 4 };
+  HWND hEd = GetDlgItem(g_sdata.hwnd, IDC_LOGWIN);
+  bool sysclr = lc >= LC_SYSCOLOR || !ReadRegSettingDW(REGCOLORIZE, true);
+  static const COLORREF clrs[] = { RGB(0, 50, 0), RGB(210, 255, 210), RGB(50, 30, 0), RGB(255, 220, 190), RGB(50, 0, 0), RGB(255, 210, 210) };
+  CHARFORMAT cf;
+  cf.cbSize = sizeof(cf), cf.dwMask = CFM_COLOR;
+  cf.dwEffects = sysclr ? CFE_AUTOCOLOR : 0;
+  cf.crTextColor = sysclr ? RGB(0, 0, 0) : clrs[(lc * 2) + 0];
+  SendMessage(hEd, em_seteditstyle, sysclr ? 0 : ses_extendbackcolor, ses_extendbackcolor);
+  SendMessage(hEd, EM_SETCHARFORMAT, 0, (LPARAM) &cf);
+  SendMessage(hEd, EM_SETBKGNDCOLOR, sysclr, sysclr ? sysclr /*Irrelevant*/ : clrs[(lc * 2) + 1]);
+}
+
 void ClearLog(HWND hwnd) {
   SetDlgItemText(hwnd, IDC_LOGWIN, _T(""));
+  SetLogColor(LC_SYSCOLOR);
   SendMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0, 0);
 }
 
@@ -260,7 +298,7 @@ void EnableDisableItems(HWND hwnd, int on)
 
   static const PACKEDCMDID_T cmds [] = {
     PACKCMDID(IDM_EXIT), PACKCMDID(IDM_LOADSCRIPT), PACKCMDID(IDM_EDITSCRIPT), 
-    PACKCMDID(IDM_SAVE), PACKCMDID(IDM_CLEARLOG),
+    PACKCMDID(IDM_SAVE), PACKCMDID(IDM_CLEARLOG), PACKCMDID(IDM_GUIDGEN),
     PACKCMDID(IDM_COMPRESSOR), PACKCMDID(IDM_COMPRESSOR_SUBMENU),
     PACKCMDID(IDM_RECOMPILE), PACKCMDID(IDM_RECOMPILE_TEST)
   };
@@ -327,7 +365,7 @@ void CompileNSISScript() {
   ClearLog(g_sdata.hwnd);
   SetTitle(g_sdata.hwnd,NULL);
   PostMessage(g_sdata.hwnd, WM_MAKENSIS_UPDATEUISTATE, 0, 0);
-  if (lstrlen(g_sdata.script)==0) {
+  if (!g_sdata.script[0]) {
     LogMessage(g_sdata.hwnd,USAGE);
     SetUIState_NoScript();
     DragAcceptFiles(g_sdata.hwnd,TRUE);
@@ -401,7 +439,7 @@ static DWORD RegReadString(HKEY hKey, LPCTSTR Name, LPTSTR Buf, DWORD cbBufSize)
   return ec;
 }
 
-static DWORD RegOpenKeyForReading(HKEY hRoot, LPCTSTR SubKey, HKEY*pKey) {
+DWORD RegOpenKeyForReading(HKEY hRoot, LPCTSTR SubKey, HKEY*pKey) {
   return RegOpenKeyEx(hRoot, SubKey, 0, KEY_READ, pKey);
 }
 
@@ -484,9 +522,9 @@ void SaveWindowPos(HWND hwnd) {
   HKEY hKey;
   WINDOWPLACEMENT p;
   p.length = sizeof(p);
-  GetWindowPlacement(hwnd, &p);
-  if (CreateRegSettingsKey(hKey)) {
-    RegSetValueEx(hKey, REGLOC, 0, REG_BINARY, (LPBYTE)&p, sizeof(p));
+  if (!GetWindowPlacement(hwnd, &p)) p.length = 0;
+  if (p.length && CreateRegSettingsKey(hKey)) {
+    RegSetValueEx(hKey, REGLOC, 0, REG_BINARY, (LPBYTE)&p, p.length);
     RegCloseKey(hKey);
   }
 }
@@ -609,8 +647,8 @@ void FreeSpawn(PROCESS_INFORMATION *pPI, HANDLE hRd, HANDLE hWr) {
   CloseHandle(hWr);
 }
 BOOL InitSpawn(STARTUPINFO &si, HANDLE &hRd, HANDLE &hWr) {
-  OSVERSIONINFO osv = {sizeof(osv)};
-  GetVersionEx(&osv);
+  OSVERSIONINFO osv;
+  GetVersionEx((osv.dwOSVersionInfoSize = sizeof(osv), &osv));
   const bool winnt = VER_PLATFORM_WIN32_NT == osv.dwPlatformId;
 
   memset(&si, 0, sizeof(STARTUPINFO));
@@ -1028,6 +1066,7 @@ HMENU FindSubMenu(HMENU hMenu, UINT uId)
 static UINT DpiGetClassicSystemDpiY() { HDC hDC = GetDC(NULL); UINT dpi = GetDeviceCaps(hDC, LOGPIXELSY); ReleaseDC(NULL, hDC); return dpi; }
 static HRESULT WINAPI DpiFallbackGetDpiForMonitor(HMONITOR hMon, int MDT, UINT*pX, UINT*pY) { return (*pX = *pY = DpiGetClassicSystemDpiY(), S_OK); }
 static UINT WINAPI DpiFallbackGetDpiForWindow(HWND hWnd) { return 0; }
+static HMONITOR WINAPI DpiFallbackMonitorFromWindow(HWND hWnd, DWORD Flags) { return NULL; }
 
 static UINT DpiNativeGetForMonitor(HMONITOR hMon)
 {
@@ -1036,9 +1075,20 @@ static UINT DpiNativeGetForMonitor(HMONITOR hMon)
   UINT x, y, mdt_effective_dpi = 0;
   return SUCCEEDED(f(hMon, mdt_effective_dpi, &x, &y)) ? y : 0; 
 }
-static UINT DpiGetForMonitor(HWND hWnd)
+UINT DpiGetForMonitor(HWND hWnd)
 {
-  HMONITOR hMon = MonitorFromWindow(hWnd, MONITOR_DEFAULTTONEAREST);
+  HMONITOR(WINAPI*monitorfromwindow)(HWND, DWORD);
+  if (SupportsWNT4() || SupportsW95())
+  {
+    static HMONITOR(WINAPI*g)(HWND, DWORD);
+    if (!g && !((FARPROC&)g = GetSysProcAddr("USER32", "MonitorFromWindow"))) g = DpiFallbackMonitorFromWindow;
+    monitorfromwindow = g;
+  }
+  else
+  {
+    monitorfromwindow = MonitorFromWindow;
+  }
+  HMONITOR hMon = monitorfromwindow(hWnd, MONITOR_DEFAULTTONEAREST);
   return hMon ? DpiNativeGetForMonitor(hMon) : (UINT)(UINT_PTR) hMon;
 }
 
@@ -1071,6 +1121,27 @@ HFONT CreateFontHelper(INT_PTR Data, int Height, DWORD p1, LPCTSTR Face)
   return CreateFont(Height, 0, 0, 0, w, FALSE, FALSE, FALSE, cs, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, paf, Face);
 }
 
+BOOL CALLBACK FontExistsCallback(const LOGFONT*pLF, const TEXTMETRIC*pTM, DWORD Type, LPARAM Cookie)
+{
+  *((BOOL*) Cookie) = TRUE;
+  return FALSE;
+}
+BOOL FontExists(LPCTSTR Face)
+{
+  BOOL ret = FALSE;
+  HDC hDC = GetDC(0);
+  EnumFonts(hDC, Face, FontExistsCallback, (LPARAM) &ret);
+  ReleaseDC(0, hDC);
+  return ret;
+}
+
+BOOL FillRectColor(HDC hDC, const RECT &Rect, COLORREF Color)
+{
+  COLORREF orgclr = SetBkColor(hDC, Color);
+  ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &Rect, _T(""), 0, NULL);
+  return TRUE|SetBkColor(hDC, orgclr);
+}
+
 static BOOL DrawHorzGradient(HDC hDC, const RECT&rect, COLOR16 r1, COLOR16 g1, COLOR16 b1, COLOR16 r2, COLOR16 g2, COLOR16 b2)
 {
   TRIVERTEX v[2] = { {rect.left, rect.top, r1, g1, b1, 0xffff}, {rect.right, rect.bottom, r2, g2, b2, 0xffff} };
@@ -1080,9 +1151,7 @@ static BOOL DrawHorzGradient(HDC hDC, const RECT&rect, COLOR16 r1, COLOR16 g1, C
   {
     if (!((FARPROC&)gf = GetSysProcAddr("MSIMG32", "GradientFill")))
     {
-      COLORREF orgclr = SetBkColor(hDC, RGB((((UINT)r1+r2)/2)>>8, (((UINT)g1+g2)/2)>>8, (((UINT)b1+b2)/2)>>8));
-      ExtTextOut(hDC, rect.left, rect.top, ETO_OPAQUE, &rect, _T(""), 0, NULL); // TODO: Actually try to draw a gradient
-      return TRUE|SetBkColor(hDC, orgclr);
+      return FillRectColor(hDC, rect, RGB((((UINT)r1+r2)/2)>>8, (((UINT)g1+g2)/2)>>8, (((UINT)b1+b2)/2)>>8)); // TODO: Actually try to draw a gradient
     }
   }
   else
