@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 2002-2017 Amir Szekely <kichik@users.sourceforge.net>
+ * Copyright (C) 2002-2021 Amir Szekely <kichik@users.sourceforge.net>
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,6 +29,8 @@
 #include <vector>
 #include <cassert>
 
+#define MAIN_ICON_LAST_IMAGE 99 // Main icon is special, we must reserve space for installer/uninstaller images
+
 #ifdef _WIN32
 #include <winnt.h>
 #else
@@ -119,53 +121,63 @@ typedef struct RESOURCE_DIRECTORY {
 
 class CResourceEditor {
 public:
-  CResourceEditor(BYTE* pbPE, int iSize, bool bKeepData = true);
+  CResourceEditor(void* pbPE, int iSize, bool bKeepData = true);
   virtual ~CResourceEditor();
+  enum { ANYLANGID = 0xffff, INVALIDLANGID = 0xffff-1, ALLLANGID = 0xffff-2 };
+  typedef enum { TM_RAW = 0, TM_ICONFILE = 0x01, TM_ICONRSRC = 0x02, TM_ICON = (TM_ICONFILE|TM_ICONRSRC), TM_AUTO = 0x04 } TYPEMANIPULATION;
 
-  // On POSIX+Unicode GetResource(RT_VERSION,..) is not TCHAR nor WINWCHAR, it is WCHAR/UINT16.
+  // On POSIX+Unicode GetResource(RT_VERSION,..) is not TCHAR nor WINWCHAR, it is WCHAR/UINT16 (MAKEINTRESOURCEW).
   // If it passes IS_INTRESOURCE we must allow it.
   // Use TCHAR* for real strings. If you need to pass in a WINWCHAR*, make GetResourceW public...
-  template<class T> bool UpdateResource(const T*Type, WORD Name, LANGID Lang, BYTE*Data, DWORD Size)
+  template<class T> bool UpdateResource(const T*Type, WORD Name, LANGID Lang, BYTE*Data, DWORD Size, TYPEMANIPULATION Manip = TM_RAW)
   {
-    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type))
-    {
-      assert(IS_INTRESOURCE(Type));
-      return false;
-    }
-    return UpdateResourceT((const TCHAR*) Type, Name, Lang, Data, Size);
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return false; }
+    return UpdateResourceT((const TCHAR*) Type, Name, Lang, Data, Size, Manip);
+  }
+  template<class T> bool UpdateResource(const T*Type, WORD Name, LANGID Lang, FILE*Data, TYPEMANIPULATION Manip = TM_AUTO)
+  {
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return false; }
+    return UpdateResourceT((const TCHAR*) Type, Name, Lang, Data, Manip);
+  }
+  template<class T> bool DeleteResource(const T*Type, WORD Name, LANGID Lang, TYPEMANIPULATION Manip = TM_RAW)
+  {
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return false; }
+    return DeleteResourceT((const TCHAR*) Type, Name, Lang, Manip);
   }
   template<class T> BYTE* GetResource(const T*Type, WORD Name, LANGID Lang)
   {
-    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type))
-    {
-      assert(IS_INTRESOURCE(Type));
-      return NULL;
-    }
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return NULL; }
     return GetResourceT((const TCHAR*) Type, Name, Lang);
   }
   template<class T> int GetResourceSize(const T*Type, WORD Name, LANGID Lang)
   {
-    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type))
-    {
-      assert(IS_INTRESOURCE(Type));
-      return -1;
-    }
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return -1; }
     return GetResourceSizeT((const TCHAR*) Type, Name, Lang);
   }
   template<class T> DWORD GetResourceOffset(const T*Type, WORD Name, LANGID Lang)
   {
-    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type))
-    {
-      assert(IS_INTRESOURCE(Type));
-      return -1;
-    }
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return -1; }
     return GetResourceOffsetT((const TCHAR*) Type, Name, Lang);
   }
+  template<class T> bool ResourceExists(const T*Type, WORD Name, LANGID Lang, LANGID*pFoundLanguage = 0)
+  {
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return false; }
+    return ResourceExistsT((const TCHAR*) Type, Name, Lang, pFoundLanguage);
+  }
+  template<class T> BYTE* GetFirstResource(const T*Type, size_t&cbData)
+  {
+    if (sizeof(T) != sizeof(TCHAR) && !IS_INTRESOURCE(Type)) { assert(IS_INTRESOURCE(Type)); return NULL; }
+    return GetFirstResourceT((const TCHAR*) Type, cbData);
+  }
 
-  bool  UpdateResourceT   (const TCHAR* szType, WORD szName, LANGID wLanguage, BYTE* lpData, DWORD dwSize);
+  bool  UpdateResourceT   (const TCHAR* szType, WORD szName, LANGID wLanguage, BYTE* lpData, DWORD dwSize, TYPEMANIPULATION Manip = TM_RAW);
+  bool  UpdateResourceT   (const TCHAR* szType, WORD szName, LANGID wLanguage, FILE*Data, TYPEMANIPULATION Manip = TM_AUTO);
+  bool  DeleteResourceT   (const TCHAR* szType, WORD szName, LANGID wLanguage, TYPEMANIPULATION Manip = TM_RAW);
   BYTE* GetResourceT      (const TCHAR* szType, WORD szName, LANGID wLanguage);
   int   GetResourceSizeT  (const TCHAR* szType, WORD szName, LANGID wLanguage);
   DWORD GetResourceOffsetT(const TCHAR* szType, WORD szName, LANGID wLanguage);
+  bool  ResourceExistsT   (const TCHAR* szType, WORD szName, LANGID wLanguage, LANGID*pFoundLanguage = 0);
+  BYTE* GetFirstResourceT (const TCHAR* szType, size_t&cbData);
   void  FreeResource(BYTE* pbResource);
 
   // The section name must be in ASCII.
@@ -182,11 +194,32 @@ public:
     DWORD *pdwResSecVA = NULL,
     DWORD *pdwSectionIndex = NULL
   );
+
+  static const TCHAR* ParseResourceTypeString(const TCHAR*String);
+  static const TCHAR* ParseResourceNameString(const TCHAR*String);
+  static LANGID ParseResourceLangString(const TCHAR*String);
+  static LANGID ParseResourceTypeNameLangString(const TCHAR**Type, const TCHAR**Name, const TCHAR*Lang);
+  static bool EditorSupportsStringNames() { return false; } // UpdateResource/GetResource do not support string names (yet)
+
 private:
-  bool  UpdateResourceW(const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage, BYTE* lpData, DWORD dwSize);
-  BYTE* GetResourceW(const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage);
-  int   GetResourceSizeW(const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage);
+  bool  UpdateResourceW   (const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage, BYTE* lpData, DWORD dwSize, TYPEMANIPULATION Manip = TM_RAW);
+  BYTE* GetResourceW      (const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage);
+  int   GetResourceSizeW  (const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage);
   DWORD GetResourceOffsetW(const WINWCHAR* szType, WINWCHAR* szName, LANGID wLanguage);
+  BYTE* GetFirstResourceW (const WINWCHAR* szType, size_t&cbData);
+  CResourceDataEntry* FindResource(const WINWCHAR* Type, const WINWCHAR* Name, LANGID Language);
+  CResourceDirectoryEntry* FindResourceLanguageDirEntryW(const WINWCHAR* Type, const WINWCHAR* Name, LANGID Language);
+  CResourceDirectoryEntry* FindResourceLanguageDirEntryT(const TCHAR* Type, const TCHAR* Name, LANGID Language);
+  bool DeleteIconImages(const CResourceDirectoryEntry& LangDir);
+  bool DeleteIconImagesW(const WINWCHAR* OwnerType, WINWCHAR* Name, LANGID LangId);
+  bool AddExtraIconFromFile(const WINWCHAR* Type, WINWCHAR* Name, LANGID LangId, BYTE* Data, DWORD Size);
+
+  BYTE* DupData(CResourceDataEntry*pDE); // Free with FreeResource
+  CResourceDirectory* ScanDirectory(PRESOURCE_DIRECTORY rdRoot, PRESOURCE_DIRECTORY rdToScan);
+  void WriteRsrcSec(BYTE* pbRsrcSec);
+  void SetOffsets(CResourceDirectory* resDir, ULONG_PTR newResDirAt);
+  DWORD AdjustVA(DWORD dwVirtualAddress, DWORD dwAdjustment);
+  DWORD AlignVA(DWORD dwVirtualAddress);
 
 private:
   BYTE* m_pbPE;
@@ -197,16 +230,7 @@ private:
 
   DWORD m_dwResourceSectionIndex;
   DWORD m_dwResourceSectionVA;
-
   CResourceDirectory* m_cResDir;
-
-  CResourceDirectory* ScanDirectory(PRESOURCE_DIRECTORY rdRoot, PRESOURCE_DIRECTORY rdToScan);
-
-  void WriteRsrcSec(BYTE* pbRsrcSec);
-  void SetOffsets(CResourceDirectory* resDir, ULONG_PTR newResDirAt);
-
-  DWORD AdjustVA(DWORD dwVirtualAddress, DWORD dwAdjustment);
-  DWORD AlignVA(DWORD dwVirtualAddress);
 };
 
 class CResourceDirectory {
@@ -243,12 +267,9 @@ public:
   bool HasName() const;
   const WINWCHAR* GetName() const;
   int GetNameLength() const;
-
   WORD GetId() const;
-
   bool IsDataDirectory() const;
   CResourceDirectory* GetSubDirectory() const;
-
   CResourceDataEntry* GetDataEntry() const;
 
   ULONG_PTR m_ulWrittenAt;
@@ -271,7 +292,6 @@ public:
   ~CResourceDataEntry();
 
   BYTE* GetData();
-
   void SetData(BYTE* pbData, DWORD dwSize);
   void SetData(BYTE* pbData, DWORD dwSize, DWORD dwCodePage);
 
