@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -61,7 +61,7 @@
 // 5000..5999 Important generic warnings
 // 6000..6999 Script warnings
 // 7000..7499 Recovered from bad input etc. warnings
-// 7500..7999 Discouraged usage warnings (allocated top to bottom to reserve as much space as possible for more bad input codes)
+// 7500..7999 Discouraged usage warnings (allocated high to low to reserve as much space as possible for more bad input codes)
 // 8000..8999 Generic warnings
 // 9000..9999 Breaking our and/or MS guidelines warnings
 typedef enum {
@@ -71,6 +71,7 @@ typedef enum {
   //DE_PP_VERBOSE_BAD_LEVEL = 4000?,
   //DW_STALE_TEMP = 5020?, TODO: There is currently no way to disable this
   DW_PACKHDR_RETNONZERO = 5021,
+  DW_BADFORMAT_EXTERNAL_FILE = 5040,
   DW_UNSUPP_STORE_FILE_ATT = 5050,
   //DW_CMDLINE_UNSUPP = 5200?,
   //DW_CMDLINE_UNSUPP_WIN = 5201?,
@@ -91,8 +92,11 @@ typedef enum {
   DW_PLUGIN_NOUNLOAD_PLACEMENT = 6080, // reserved ..6099
   DW_PP_PRAGMA_UNKNOWN = 6100, // reserved ..6199
   DW_PP_PRAGMA_INVALID = 6101,
+  DW_PP_DELFILE_DELERROR = 6149,
+  DW_PP_DELFILE_NOMATCH = DW_PP_DELFILE_DELERROR,
   DW_PP_VERBOSE_POP_EMPTY_STACK = 6150,
   //DW_PP_VERBOSE_BAD_LEVEL = 6151?, // 2.x failed to issue a warning. 3.x currently aborts with hard error.
+  DW_PP_UNDEF_UNDEFINED = 6155,
   DW_INCLUDE_NONFATAL_NOT_FOUND = 7000, // reserved ..7009
   DW_FILE_NONFATAL_NOT_FOUND = 7010, // reserved ..7019
   DW_LANGSTRING_OVERLONGLENGTH = 7020, // reserved ..7024
@@ -104,7 +108,9 @@ typedef enum {
   DW_LICENSE_EMPTY = 7050,
   DW_ATTRIBUTE_OVERLONGSTRING = 7060,
   DW_PARSE_BADNUMBER = 7070,
+  DW_PARSE_NUMBEROUTOFSPEC = 7071,
   DW_PARSE_LNK_HK = 7075,
+  DW_GENERIC_DEPRECATED = 7998,
   DW_PARSE_REGPATHPREFIX = 7999,
   DW_INSTFILESPAGE_NOT_USED = 8000, // reserved ..8019
   DW_COMP_FINAL = 8020, // reserved ..8059
@@ -139,11 +145,12 @@ namespace MakensisAPI {
   extern const TCHAR* SigintEventNameFmt;
   extern const TCHAR* SigintEventNameLegacy;
 
-  enum notify_e {
-    NOTIFY_SCRIPT, // main nsi file(s)
+  enum datatransfer_e {
+    NOTIFY_SCRIPT, // Compiler -> Host: main nsi file(s)
     NOTIFY_WARNING,
     NOTIFY_ERROR,
-    NOTIFY_OUTPUT // generated .exe file
+    NOTIFY_OUTPUT, // Compiler -> Host: Generated .exe file
+    PROMPT_FILEPATH // [0x03006000] Compiler -> Host -> Compiler
   };
 #ifdef _WIN32
   enum sndmsg_e {
@@ -152,8 +159,14 @@ namespace MakensisAPI {
 #endif
   enum QUERYHOST_e {
     QH_OUTPUTCHARSET = 1, // [0x03000000] return (wincodepage+1) or 0 for default (This encoding is used by stdout, stderr and the notify messages)
-    QH_ENABLESTDERR // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_ENABLESTDERR, // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_SUPPORTEDVERSION, // [0x03006000] The host must return new highest makensis compiler version it supports
   };
+  typedef struct {
+    unsigned char Platform; // Bitness OR'ed with sizeof(TCHAR) of the compiler
+    unsigned char Reserved;
+    TCHAR Path[1];
+  } PROMPT_FILEPATH_DATA;
 }
 
 #define PAGE_CUSTOM 0
@@ -167,18 +180,13 @@ namespace MakensisAPI {
 #define FLAG_OFFSET(flag) (FIELD_OFFSET(exec_flags_t, flag)/sizeof(int))
 
 class DiagState {
-  template<class M> struct mapped_type_helper { typedef typename M::value_type::second_type type; }; // VC6 uses referent_type and not mapped_type
   template<class C, class K, class V> void insert_or_assign(C&c, const K&k, V val)
   {
     typename C::value_type item(k, val);
-#if defined(_MSC_VER) && _MSC_VER <= 1200
-    std::pair<C::iterator, bool> ret = c.insert(item);
-#else
-    std::pair<typename C::iterator, bool> ret = c.insert(item);
-#endif
+    std::pair<NSIS_CXX_TYPENAME C::iterator, bool> ret = c.insert(item);
     if (!ret.second) ret.first->second = val;
   }
-  template<class C, class K> typename mapped_type_helper<C>::type get_paired_value(const C&c, const K&k, typename mapped_type_helper<C>::type defval) const
+  template<class C, class K> typename STL::mapped_type<C>::type get_paired_value(const C&c, const K&k, typename STL::mapped_type<C>::type defval) const
   {
     typename C::const_iterator it = c.find(k);
     return c.end() == it ? defval : it->second;
@@ -230,6 +238,7 @@ class CEXEBuild {
       TARGET_X86ANSI = TARGETFIRST,
       TARGET_X86UNICODE,
       TARGET_AMD64, // Always Unicode
+      TARGET_ARM64, // Always Unicode
       TARGET_UNKNOWN,
       TARGETCOUNT = (TARGET_UNKNOWN-TARGETFIRST)
     } TARGETTYPE;
@@ -238,7 +247,7 @@ class CEXEBuild {
     bool m_previous_x86_unicode;
     const TCHAR* get_target_suffix(CEXEBuild::TARGETTYPE tt, const TCHAR*defval = _T("?")) const;
     const TCHAR* get_target_suffix() const { return get_target_suffix(m_target_type); }
-    static bool is_targettype_64bit(TARGETTYPE tt) { return TARGET_AMD64 == tt; }
+    static bool is_targettype_64bit(TARGETTYPE tt) { return TARGET_AMD64 == tt || TARGET_ARM64 == tt; }
     bool is_target_64bit() const { return is_targettype_64bit(m_target_type); }
     void print_bad_targettype_parameter(const TCHAR*cmdname, const TCHAR*prefix = _T("")) const;
     unsigned int get_header_size() const { return (unsigned int)sizeof(header) + (is_target_64bit() ? (4 * BLOCKS_NUM) : 0); }
@@ -254,6 +263,7 @@ class CEXEBuild {
     int write_output(void);
 
     void print_help(const TCHAR *commandname=NULL);
+    bool print_cmdhelp(const TCHAR *commandname, bool cmdhelp=false);
 
     DefineList definedlist; // List of identifiers marked as "defined" like
                             // C++ macro definitions such as _UNICODE.
@@ -272,7 +282,14 @@ class CEXEBuild {
     NStreamLineReader* curlinereader;
 
     HWND notify_hwnd;
-    void notify(MakensisAPI::notify_e code, const TCHAR *data) const;
+    void notify(MakensisAPI::datatransfer_e code, const TCHAR *data) const;
+    #ifdef _WIN32
+    typedef bool (CALLBACK*HOSTAPIREQUESTDATAPROC)(void*cookie, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
+    #else
+    typedef bool (*HOSTAPIREQUESTDATAPROC)(void*cookie);
+    #endif
+    bool hostapi_request_data(MakensisAPI::datatransfer_e operation, UINT minver, HOSTAPIREQUESTDATAPROC proc, void*cookie, const void* input, size_t inputsize) const;
+    bool prompt_for_output_path(TCHAR*path, UINT pathcap) const;
 
   private:
     int check_write_output_errors() const;
@@ -326,6 +343,7 @@ class CEXEBuild {
     int doParse(const TCHAR *str);
     int doCommand(int which_token, LineParser &line);
     TCHAR m_templinebuf[MAX_LINELENGTH]; // Buffer used by parseScript() & doCommand(), not recursion safe!
+    int add_flag_instruction_entry(int which_token, int opcode, LineParser &line, int offset, int data = 0);
 
     int do_add_file(const TCHAR *lgss, int attrib, int recurse, int *total_files, const TCHAR 
       *name_override=0, int generatecode=1, int *data_handle=0, 
@@ -441,6 +459,7 @@ class CEXEBuild {
     void print_warnings();
     void warninghelper(DIAGCODE dc, bool fl, const TCHAR *fmt, va_list args);
     DiagState diagstate;
+    bool changed_target;
 
     /** Are we defining an uninstall version of the code?
      * @param un Use like a boolean to define whether in uninstall mode.
@@ -506,7 +525,7 @@ class CEXEBuild {
      * a PS_ERROR.  If this function call is overwriting a set user string,
      * this will return a PS_WARNING.
      */
-    int SetInnerString(int id, TCHAR *str);
+    int SetInnerString(int id, const TCHAR *str);
 
     int GenerateLangTable(LanguageTable *lt, int num_lang_tables);
     int GenerateLangTables();
@@ -550,6 +569,7 @@ class CEXEBuild {
     int DeclaredUserVar(const TCHAR *VarName);
     void VerifyDeclaredUserVarRefs(UserVarsStringList *pVarsStringList);
     bool IsIntOrUserVar(const LineParser &line, int token) const;
+    static bool IsVarPrefix(const TCHAR*s) { return *s++ == _T('$') && *s > ' '; }
 
     ConstantsStringList m_ShellConstants;
 
@@ -583,7 +603,10 @@ class CEXEBuild {
       struct postbuild_cmd*next;
       int cmpop, cmpval;
       TCHAR cmd[1];
+      void delete_all();
+      static postbuild_cmd* make(const TCHAR *cmdstr, int cmpop, int cmpval);
     } *postbuild_cmds;
+    int run_postbuild_cmds(const postbuild_cmd *cmds, const TCHAR *templatearg_pc1, const TCHAR* commandname);
     int check_external_exitcode(int exitcode, int op, int val);
 
     TCHAR build_packname[1024], build_packcmd[1024];
@@ -673,11 +696,14 @@ class CEXEBuild {
 #endif
 
     WORD PEDllCharacteristics, PESubsysVerMaj, PESubsysVerMin;
+    unsigned int manifest_flags;
     manifest::comctl manifest_comctl;
     manifest::exec_level manifest_exec_level;
     manifest::dpiaware manifest_dpiaware;
     tstring manifest_dpiawareness;
+    manifest::longpathaware manifest_lpaware;
     manifest::SupportedOSList manifest_sosl;
+    tstring manifest_maxversiontested;
 
     CResourceEditor *res_editor;
     void init_res_editor();
