@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft, Jeff Doozan and Contributors
+ * Copyright (C) 1999-2021 Nullsoft, Jeff Doozan and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -1236,18 +1236,19 @@ static void FORCE_INLINE NSISCALL RefreshComponents(HWND hwTree, HTREEITEM *item
     TreeView_SetItem(hwTree, &item);
   }
 
-  // workaround for bug #1397031
+  // workaround for bug #1397031 A.K.A #434
   //
-  // windows 95 doesn't erase the background of the state image
-  // before it draws a new one. because of this parts of the old
+  // Windows 95 & NT4 doesn't erase the background of the state image
+  // before it draws a new one. Because of this parts of the old
   // state image will show where the new state image is masked.
   //
-  // to solve this, the following line forces the background to
-  // be erased. sadly, this redraws the entire control. it might
+  // To solve this, the following line forces the background to
+  // be erased. sadly, this redraws the entire control. It might
   // be a good idea to figure out where the state images are and
   // redraw only those.
 
-  InvalidateRect(hwTree, NULL, TRUE);
+  if (IsWin95NT4()) // Checking for < IE4 is probably better but more work
+    InvalidateRect(hwTree, NULL, TRUE);
 }
 
 int NSISCALL TreeGetSelectedSection(HWND tree, BOOL mouse)
@@ -1342,7 +1343,7 @@ static INT_PTR CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM l
 
     hTreeItems=(HTREEITEM*)GlobalAlloc(GPTR,sizeof(HTREEITEM)*num_sections);
 
-    hBMcheck1=LoadBitmap(g_hInstance, MAKEINTRESOURCE(IDB_BITMAP1));
+    hBMcheck1=LoadImage(g_hInstance, MAKEINTRESOURCE(IDB_BITMAP1), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR); // LR_CREATEDIBSECTION required to load TopDown bitmaps but that breaks modern.bmp
 
     last_selected_tree_item=-1;
     oldTreeWndProc=(WNDPROC)SetWindowLongPtr(hwndTree1,GWLP_WNDPROC,(LONG_PTR)newTreeWndProc);
