@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2017 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -176,33 +176,42 @@ static HKEY NSISCALL RegCreateScriptKey(int RootKey, LPCTSTR SubKey, REGSAM RS)
 // RegDeleteKeyEx on 32-bit Windows accepts but ignores the KEY_WOW64_xxKEY flags and always uses the 
 // one and only native key. Our RegKeyOpen will intentionally fail if a incompatible WoW64 flag is used.
 #define DRTF_ONLYIFNOSUBKEYS DELREGKEY_ONLYIFNOSUBKEYS
+#define DRTF_ONLYIFNOVALUES  DELREGKEY_ONLYIFNOVALUES
 static LONG NSISCALL DeleteRegTree(HKEY hThisKey, LPCTSTR SubKey, REGSAM SamviewAndFlags)
 {
   HKEY hKey;
-  UINT onlyifempty = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnosubkeys = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnovalues  = SamviewAndFlags & DRTF_ONLYIFNOVALUES, valuesexistcheckinsubkeys = TRUE;
   REGSAM samview = SamviewAndFlags & (KEY_WOW64_32KEY|KEY_WOW64_64KEY);
-  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|samview, &hKey);
+  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|KEY_QUERY_VALUE|samview, &hKey);
   if (retval == ERROR_SUCCESS)
   {
     TCHAR child[MAX_PATH+1]; // NB - don't change this to static (recursive function)
+    if (onlyifnovalues)
+    {
+      DWORD cchName = 0;
+      retval = RegEnumValue(hKey, 0, child, &cchName, NULL, NULL, NULL, NULL);
+      if (retval != ERROR_NO_MORE_ITEMS) goto notempty;
+      if (!valuesexistcheckinsubkeys) SamviewAndFlags &= ~DRTF_ONLYIFNOVALUES;
+    }
     while (RegEnumKey(hKey, 0, child, COUNTOF(child)) == ERROR_SUCCESS)
     {
-      if (onlyifempty) return (RegCloseKey(hKey), !ERROR_SUCCESS);
+      if (onlyifnosubkeys) notempty: return (RegCloseKey(hKey), ERROR_CAN_NOT_COMPLETE);
       if ((retval = DeleteRegTree(hKey, child, SamviewAndFlags)) != ERROR_SUCCESS) break;
     }
     RegCloseKey(hKey);
     {
-      typedef LONG (WINAPI * RegDeleteKeyExPtr)(HKEY, LPCTSTR, REGSAM, DWORD);
-      RegDeleteKeyExPtr RDKE = (RegDeleteKeyExPtr)
-      #ifdef _WIN64
-        RegDeleteKeyEx;
-      #else
+      typedef LONG (WINAPI * RegDeleteKeyExType)(HKEY, LPCTSTR, REGSAM, DWORD);
+      RegDeleteKeyExType RDKE = (RegDeleteKeyExType)
+      #if !defined(_WIN64) || defined(_M_IA64)
         myGetProcAddress(MGA_RegDeleteKeyEx);
-      #endif
-      if (RDKE)
-        retval = RDKE(hThisKey, SubKey, samview, 0);
-      else
+      if (!RDKE)
         retval = RegDeleteKey(hThisKey, SubKey);
+      else
+      #else
+        RegDeleteKeyEx;
+      #endif
+        retval = RDKE(hThisKey, SubKey, samview, 0);
     }
   }
   return retval;
@@ -210,6 +219,7 @@ static LONG NSISCALL DeleteRegTree(HKEY hThisKey, LPCTSTR SubKey, REGSAM Samview
 static LONG NSISCALL RegDeleteScriptKey(int RootKey, LPCTSTR SubKey, REGSAM SamviewAndFlags)
 {
   HKEY hKey;
+  if (!SubKey[0]) return ERROR_CAN_NOT_COMPLETE; // Don't allow scripts to delete a HKEY root
   SamviewAndFlags |= KEY_FROMSCRIPT;
   hKey = GetRegKeyAndSAM(GetRegRootKey(RootKey), &SamviewAndFlags);
   return hKey ? DeleteRegTree(hKey, SubKey, SamviewAndFlags) : ERROR_INVALID_HANDLE; // ERROR_CANTOPEN?
@@ -683,18 +693,27 @@ static int NSISCALL ExecuteEntry(entry *entry_)
 #ifdef NSIS_SUPPORT_INTOPTS
     case EW_INTCMP:
       {
-        int v,v2;
-        v=GetIntFromParm(0);
-        v2=GetIntFromParm(1);
-        if (!parm5) {
-          // signed
-          if (v<v2) return parm3;
-          if (v>v2) return parm4;
+        UINT supp64=sizeof(void*) > 4, opu=supp64 ? (BYTE) parm5 : parm5, op64=supp64 ? (SHORT) parm5 < 0 : FALSE;
+        INT_PTR v=GetIntPtrFromParm(0), v2=GetIntPtrFromParm(1); // Note: This needs to be INT64 if supp64 is ever set to true for 32-bit builds!
+        if (!opu) { // signed:
+          if (op64) {
+            if ((INT64)v < (INT64)v2) return parm3;
+            if ((INT64)v > (INT64)v2) return parm4;
+          }
+          else {
+            if ((signed int)v < (signed int)v2) return parm3;
+            if ((signed int)v > (signed int)v2) return parm4;
+          }
         }
-        else {
-          // unsigned
-          if ((unsigned int)v<(unsigned int)v2) return parm3;
-          if ((unsigned int)v>(unsigned int)v2) return parm4;
+        else { // unsigned:
+          if (op64) {
+            if ((UINT64)v < (UINT64)v2) return parm3;
+            if ((UINT64)v > (UINT64)v2) return parm4;
+          }
+          else {
+            if ((unsigned int)v < (unsigned int)v2) return parm3;
+            if ((unsigned int)v > (unsigned int)v2) return parm4;
+          }
         }
       }
     return parm2;
@@ -702,7 +721,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
       {
         int v,v2;
         TCHAR *p=var0;
-        v=GetIntFromParm(1); // BUGBUG64: TODO: These should be INT_PTR, the script might be playing with pointers and System::Call
+        v=GetIntFromParm(1);
         v2=GetIntFromParm(2);
         switch (parm3)
         {
@@ -719,15 +738,15 @@ static int NSISCALL ExecuteEntry(entry *entry_)
           case 10: if (v2) v%=v2; else { v=0; exec_error++; } break;
           case 11: v=v<<v2; break;
           case 12: v=v>>v2; break;
+          case 13: v=(unsigned int)v>>(unsigned int)v2; break;
         }
         myitoa(p,v);
       }
     break;
     case EW_INTFMT: {
       TCHAR *buf0=GetStringFromParm(0x01);
-      wsprintf(var0,
-               buf0,
-               GetIntPtrFromParm(2));
+      INT_PTR val=GetIntPtrFromParm(2), op64=sizeof(void*) > 4 && parm3;
+      wsprintf(var0,buf0,op64 ? val : (UINT)val);
     }
     break;
 #endif//NSIS_SUPPORT_INTOPTS
@@ -821,28 +840,18 @@ static int NSISCALL ExecuteEntry(entry *entry_)
       SetWindowLongPtr(GetHwndFromParm(0), GWLP_USERDATA, (LONG_PTR) c);
     }
     break;
-    case EW_SETBRANDINGIMAGE:
+    case EW_LOADANDSETIMAGE:
     {
       RECT r;
-      HANDLE hImage;
-      HWND hwImage=GetDlgItem(g_hwnd, parm1);
-      GetClientRect(hwImage, &r);
-      hImage=LoadImage(
-        0,
-        GetStringFromParm(0x00),
-        IMAGE_BITMAP,
-        parm2*r.right,
-        parm2*r.bottom,
-        LR_LOADFROMFILE
-      );
-      hImage = (HANDLE)SendMessage(
-        hwImage,
-        STM_SETIMAGE,
-        IMAGE_BITMAP,
-        (LPARAM)hImage
-      );
-      // delete old image
-      if (hImage) DeleteObject(hImage);
+      HANDLE hNewImage, hPrevImage;
+      HWND hCtl=(parm3 & LASIF_HWND) ? GetHwndFromParm(2) : GetDlgItem(g_hwnd, parm2);
+      UINT it=parm3 & LASIM_IMAGE, exeres=parm3 & LASIF_EXERES, fitw=(UINT)parm3 >> LASIS_FITCTLW, fith=(parm3 & LASIF_FITCTLH) != 0;
+      LPCTSTR imgid = (parm3 & LASIF_STRID) ? GetStringFromParm(0x11) : MAKEINTRESOURCE(parm1);
+      GetClientRect(hCtl, &r);
+      hNewImage=LoadImage(exeres ? g_hInstance : NULL, imgid, it, fitw*r.right, fith*r.bottom, parm3 & LASIM_LR);
+      hPrevImage=(HANDLE)SendMessage(hCtl, STM_SETIMAGE, it, (LPARAM)hNewImage);
+      if (hPrevImage && IMAGE_BITMAP == it) DeleteObject(hPrevImage); // Delete the old bitmap
+      if (parm0 >= 0) iptrtostr(var0, (INT_PTR)hNewImage); // Optional output handle
     }
     break;
     case EW_CREATEFONT:
@@ -1391,7 +1400,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         else if (which==EW_FPUTS)
         {
           GetStringFromParm(0x21); // load string in buf2, convert it to ANSI in buf1
-          WideCharToMultiByte(CP_ACP, 0, buf2, -1, (LPSTR) buf1, NSIS_MAX_STRLEN, NULL, NULL);
+          StrWideToACP(buf2, (LPSTR) buf1, NSIS_MAX_STRLEN);
           l=lstrlenA((LPCSTR)buf1);
         }
 #endif
@@ -1705,6 +1714,32 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     break;
 #endif//NSIS_CONFIG_COMPONENTPAGE
 
+    case EW_GETOSINFO:
+    {
+      //switch(parm0)
+      {
+#ifdef NSIS_SUPPORT_FNUTIL
+      //case 0:
+        {
+          TCHAR *outstr = var1;
+          IID kfid;
+          HRESULT(WINAPI*SHGKFP)(REFIID,DWORD,HANDLE,PWSTR*) = (HRESULT(WINAPI*)(REFIID,DWORD,HANDLE,PWSTR*)) myGetProcAddress(MGA_SHGetKnownFolderPath);
+          TCHAR *buf2 = GetStringFromParm(0x22), succ = FALSE;
+          if (SHGKFP && SUCCEEDED(ComIIDFromString(buf2, &kfid)))
+          {
+            PWSTR path;
+            HRESULT hr = SHGKFP(&kfid, parm3, NULL, &path);
+            if (SUCCEEDED(hr))
+              strcpyWideToT(outstr, path), CoTaskMemFree(path), ++succ;
+          }
+          if (!succ)
+            exec_error++, *outstr = _T('\0');
+        }
+        //break;
+#endif
+      }
+    }
+    break;
 #ifdef NSIS_LOCKWINDOW_SUPPORT
     case EW_LOCKWINDOW:
     {
