@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2017 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -96,9 +96,9 @@ enum
   EW_READENVSTR,        // ReadEnvStr/ExpandEnvStrings: 3 [output, string_with_env_variables, IsRead]
 #endif
 #ifdef NSIS_SUPPORT_INTOPTS
-  EW_INTCMP,            // IntCmp: 6 [val1, val2, equal, val1<val2, val1>val2, unsigned?]
-  EW_INTOP,             // IntOp: 4 [output, input1, input2, op] where op: 0=add, 1=sub, 2=mul, 3=div, 4=bor, 5=band, 6=bxor, 7=bnot input1, 8=lnot input1, 9=lor, 10=land], 11=1%2
-  EW_INTFMT,            // IntFmt: [output, format, input]
+  EW_INTCMP,            // IntCmp: 6 [val1, val2, equal, val1<val2, val1>val2, flags] where flags: bit 0x01 is set for unsigned operations and bit 0x8000 is set for 64-bit operations
+  EW_INTOP,             // IntOp: 4 [output, input1, input2, op] where op: 0=add, 1=sub, 2=mul, 3=div, 4=bor, 5=band, 6=bxor, 7=bnot input1, 8=lor, 9=land 10=mod, 11=shl, 12=sar, 13=shr (bneg is implemented with bxor in compiler)
+  EW_INTFMT,            // IntFmt: 4 [output, format, input, 64-bit]
 #endif
 #ifdef NSIS_SUPPORT_STACK
   EW_PUSHPOP,           // Push/Pop/Exchange: 3 [variable/string, ?pop:push, ?exch]
@@ -112,7 +112,7 @@ enum
 #ifdef NSIS_CONFIG_ENHANCEDUI_SUPPORT
   EW_GETDLGITEM,        // GetDlgItem:        3: [outputvar, dialog, item_id]
   EW_SETCTLCOLORS,      // SerCtlColors:      3: [hwnd, pointer to struct colors]
-  EW_SETBRANDINGIMAGE,  // SetBrandingImage:  1: [Bitmap file]
+  EW_LOADANDSETIMAGE,   // SetBrandingImage/LoadAndSetImage: 5: [ctrl imagetype lrflags imageid [output]]
   EW_CREATEFONT,        // CreateFont:        5: [handle output, face name, height, weight, flags]
   EW_SHOWWINDOW,        // ShowWindow:        2: [hwnd, show state]
 #endif
@@ -193,9 +193,8 @@ enum
                         // InstTypeGetFlags:  3: [idx, 1, output]
 #endif
 
-  // instructions not actually implemented in exehead, but used in compiler.
-  EW_GETLABELADDR,      // both of these get converted to EW_ASSIGNVAR
-  EW_GETFUNCTIONADDR,
+  EW_GETOSINFO,         // 1+ [operation, ...]
+  EW_RESERVEDOPCODE,    // Free slot, feel free to use it for something
 
 #ifdef NSIS_LOCKWINDOW_SUPPORT
   EW_LOCKWINDOW,
@@ -207,6 +206,10 @@ enum
   EW_FGETWS,            // FileReadUTF16LE: 4 [handle, output, maxlen, ?getchar:gets]
 #endif//NSIS_SUPPORT_FILEFUNCTIONS
 #endif
+
+  // Opcodes listed here are not actually used in exehead. No exehead opcodes should be present after these!
+  EW_GETLABELADDR,      // --> EW_ASSIGNVAR
+  EW_GETFUNCTIONADDR,   // --> EW_ASSIGNVAR
 };
 
 #pragma pack(push, 1) // fileform.cpp assumes no padding/alignment
@@ -467,12 +470,27 @@ typedef struct
   int parms[5];
 } page;
 
-// text/bg color
+
+// EW_LOADANDSETIMAGE flags, masks and shifts
+#define LASIS_FITCTLW 31 // Top bit because it MUST shift to a value of 0 or 1
+#define LASIF_FITCTLW ( (unsigned int)1 << LASIS_FITCTLW )
+#define LASIF_FITCTLH ( (unsigned int)1 << 30 )
+#define LASIM_IMAGE  0x00000003 // IMAGE_*
+#define LASIF_EXERES 0x00000004 // GetModuleHandle(NULL).
+#define LASIF_HWND   0x00000100 // Don't call GetDlgItem.
+#define LASIF_STRID  0x00010000
+#define LASIM_LR   ( 0x0000fff0 & ~(LASIM_IMAGE|LASIF_EXERES|LASIF_HWND|LASIF_STRID) )
+#define LASIF_LR_LOADFROMFILE 0x00000010
+
+
+// ctlcolors text/bg color flags
 #define CC_TEXT 1
 #define CC_TEXT_SYS 2
 #define CC_BK 4
 #define CC_BK_SYS 8
 #define CC_BKB 16
+#define CC_FLAGSMASK 0x1f
+#define CC_FLAGSSHIFTFORZERO 5
 
 typedef struct {
   COLORREF text;
@@ -514,7 +532,7 @@ typedef struct {
 
 #define REGROOTVIEW32 0x40000000
 #define REGROOTVIEW64 0x20000000
-#define REGROOTVIEWTOSAMVIEW(rv) ( ((UINT_PTR)(rv)&(REGROOTVIEW32|REGROOTVIEW64)) >> 21 ) // REGROOTVIEWxx to KEY_WOW64_xxKEY
+#define REGROOTVIEWTOSAMVIEW(rv) ( (UINT)(((UINT_PTR)(rv)&(REGROOTVIEW32|REGROOTVIEW64)) >> 21) ) // REGROOTVIEWxx to KEY_WOW64_xxKEY
 #define IsRegRootkeyForcedView(hKey) ( ((UINT_PTR) (hKey) & (REGROOTVIEW32|REGROOTVIEW64)) )
 #define MAKEREGROOTVIEW(r, fv) ( (HKEY) ((UINT_PTR)(r) | (fv)) )
 #define HKSHCTX ( (HKEY) 0 ) // Converted to HKCU or HKLM by GetRegRootKey
@@ -532,8 +550,10 @@ typedef struct {
 #define HKLMANY MAKEREGROOTVIEW(HKEY_LOCAL_MACHINE, REGROOTVIEW32|REGROOTVIEW64)
 #define DELREG_VALUE 0 // TOK_DELETEREGVALUE
 #define DELREG_KEY 1 // TOK_DELETEREGKEY
-#define DELREGKEY_ONLYIFNOSUBKEYS 1 // Shifted and stored as 2 in the binary for compatibility with <= 3.1
-#define DELREGKEYFLAGSSHIFT 1 // parm4 is shifted so exehead can remove the DELREG_KEY bit
+#define DELREGKEY_ONLYIFNOSUBKEYS 0x01 // Note: Shifted (stored as 2 in the binary) for compatibility with <= v3.1
+#define DELREGKEY_ONLYIFNOVALUES  0x02
+//      DELREGKEY_SAMVIEWMASK     REGROOTVIEWTOSAMVIEW(REGROOTVIEW32|REGROOTVIEW64) // Reserved for KEY_WOW64_xxKEY, cannot be used as flags!
+#define DELREGKEYFLAGSSHIFT 1 // exehead removes the DELREG_KEY bit in parm4 by shifting. After shifting the bits are DELREGKEY_*.
 
 
 #ifdef NSIS_SUPPORT_CREATESHORTCUT
@@ -569,10 +589,21 @@ typedef struct {
 
 #pragma pack(pop)
 
-#ifdef EXEHEAD
+#define NSIS_MAX_EXEDATASIZE 0x7fffffffUL // Maximum size of .exe including compressed installer data.
+#ifndef NSIS_CONFIG_CRC_ANAL
+#define NSIS_MAX_EXEFILESIZE 0xffffffffUL // Maximum size of .exe including compressed installer data AND 3rd-party appended data. (Windows refuses to run .EXE files larger than 4 GiB)
+#else
+#define NSIS_MAX_EXEFILESIZE NSIS_MAX_EXEDATASIZE
+#endif
 
+#ifdef EXEHEAD
 // the following are only used/implemented in exehead, not makensis.
 
+#if NSIS_MAX_EXEDATASIZE <= 0xffffffffUL
+#define MAXEXEDATASIZETYPE UINT // Maximum size of .exe including compressed installer data. (Unsigned allows size including 3rd-party appeded data to be 4 GiB instead of 2 GiB)
+#endif
+#define MAXSIZETYPE UINT
+
 int NSISCALL isheader(firstheader *h); // returns 0 on not header, length_of_datablock on success
 
 // returns nonzero on error
@@ -595,7 +626,7 @@ DWORD NSISCALL SetSelfFilePointer(LONG lDistanceToMove);
 extern struct block_header g_blocks[BLOCKS_NUM];
 extern header *g_header;
 extern int g_flags;
-extern int g_filehdrsize;
+extern UINT g_filehdrsize;
 extern int g_is_uninstaller;
 
 #define g_pages ( (page*) g_blocks[NB_PAGES].offset )
