@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2017 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -32,12 +32,12 @@
 #include "winchar.h" // assert(sizeof(WINWCHAR)...)
 
 #include <nsis-version.h>
-#define NSIS_COPYYEARS _T("1999-2017")
+#define NSIS_COPYYEARS _T("1999-2021")
 
 using namespace std;
 
 NSISRT_DEFINEGLOBALS();
-bool g_dopause=false, g_warnaserror=false;
+bool g_dopause=false;
 NStreamEncoding g_outputenc;
 #ifdef _WIN32
 UINT g_wincon_orgoutcp;
@@ -70,8 +70,10 @@ static void myatexit()
 {
   dopause();
   ResetPrintColor();
-  if (g_output != stdout && g_output) fclose(g_output), g_output = 0;
-  if (g_errout != stderr && g_errout) fclose(g_errout), g_errout = 0;
+  bool oneoutputstream = g_output == g_errout;
+  if (g_output != stdout && g_output                    ) fclose(g_output);
+  if (g_errout != stderr && g_errout && !oneoutputstream) fclose(g_errout);
+  g_output = g_errout = 0;
 #ifdef _WIN32
   SetConsoleOutputCP(g_wincon_orgoutcp);
 #endif
@@ -171,6 +173,7 @@ static void print_usage()
          _T("  ") OPT_STR _T("HDRINFO prints information about what options makensis was compiled with\n")
          _T("  ") OPT_STR _T("LICENSE prints the makensis software license\n")
          _T("  ") OPT_STR _T("VERSION prints the makensis version and exits\n")
+         //_T("  ") OPT_STR _T("HELP this usage info\n")
 #ifdef _WIN32
          _T("  ") OPT_STR _T("Px sets the compiler process priority, where x is 5=realtime,4=high,\n")
          _T("  ")         _T("  3=above normal,2=normal,1=below normal,0=idle\n")
@@ -307,7 +310,7 @@ static inline int makensismain(int argc, TCHAR **argv)
   NStreamEncoding inputenc, &outputenc = g_outputenc;
   int argpos=0;
   bool do_cd=true, noconfig=false;
-  bool no_logo=true;
+  bool no_logo=true, warnaserror=false;
   bool initialparsefail=false, in_files=false;
   bool oneoutputstream=false;
   signed char pponly=0;
@@ -377,7 +380,7 @@ static inline int makensismain(int argc, TCHAR **argv)
     }
     else if (!_tcsicmp(swname,_T("WX")))
     {
-      g_warnaserror = true;
+      warnaserror = true;
     }
     // This must be parsed last because it will eat other switches
     else if (S7IsChEqualI('o',swname[0]) && swname[1]) stdoutredirname=swname+1;
@@ -419,11 +422,11 @@ static inline int makensismain(int argc, TCHAR **argv)
   // g_output is now initialized and Print*/_[f]tprintf can be used
   if (!stdoutredir) PrintColorFmtMsg_WARN(_T("Error opening output log for writing! Using stdout.\n"));
 
-  unsigned int nousage=0;
+  unsigned int nousage=0, performed=0;
   unsigned int files_processed=0;
   unsigned int cmds_processed=0;
 
-  CEXEBuild build(pponly);
+  CEXEBuild build(pponly, warnaserror);
   try
   {
     build.initialize(argv[0]);
@@ -466,23 +469,25 @@ static inline int makensismain(int argc, TCHAR **argv)
       else if (!_tcsicmp(swname,_T("NOCD"))) do_cd=false;
       else if (!_tcsicmp(swname,_T("NOCONFIG"))) noconfig=true;
       else if (!_tcsicmp(swname,_T("PAUSE"))) g_dopause=true;
+      else if (!_tcsicmp(swname,_T("HELP")))
+      {
+        print_usage();
+        performed |= ++nousage;
+      }
       else if (!_tcsicmp(swname,_T("LICENSE"))) 
       {
         if (build.display_info) print_license();
-        nousage++;
+        performed |= ++nousage;
       }
       else if (!_tcsicmp(swname,_T("CMDHELP")))
       {
-        if (argpos < argc-1)
-          build.print_help(argv[++argpos]);
-        else
-          build.print_help(NULL);
-        nousage++;
+        if (build.print_cmdhelp(argpos < argc-1 ? argv[++argpos] : NULL, true))
+          performed |= ++nousage;
       }
       else if (!_tcsicmp(swname,_T("HDRINFO")))
       {
         print_stub_info(build);
-        nousage++;
+        performed |= ++nousage;
       }
       else if (!_tcsicmp(swname,_T("INPUTCHARSET")) || !_tcsicmp(swname,_T("ICS")))
       {
@@ -639,13 +644,14 @@ static inline int makensismain(int argc, TCHAR **argv)
     argpos++;
   }
 
-  if (argpos < argc || (!files_processed && !cmds_processed))
+  bool parsed_all_params = argpos >= argc, processed_any = files_processed || cmds_processed;
+  if (!parsed_all_params || !processed_any)
   {
     if (build.display_errors && !nousage)
     {
       print_usage();
     }
-    return 1;
+    return performed && parsed_all_params ? 0 : 1;
   }
 
   if (build.preprocessonly) return 0;
@@ -679,7 +685,8 @@ int _tmain(int argc, TCHAR **argv)
 {
 #ifndef NDEBUG
 #ifdef _MSC_VER
-  _CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) | _CRTDBG_CHECK_ALWAYS_DF);
+  const int dbgchkthorough = true, checkeveryallocfree = dbgchkthorough ? _CRTDBG_CHECK_ALWAYS_DF : 0;
+  _CrtSetDbgFlag(_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG) | _CRTDBG_ALLOC_MEM_DF | checkeveryallocfree | _CRTDBG_LEAK_CHECK_DF);
   _CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE), _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);
   _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE), _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDOUT);
   //_CrtSetReportMode(_CRT_ASSERT, _CRTDBG_MODE_FILE), _CrtSetReportFile(_CRT_ASSERT, _CRTDBG_FILE_STDOUT);
@@ -688,7 +695,6 @@ int _tmain(int argc, TCHAR **argv)
   int retval = makensismain(argc,argv);
 #ifndef NDEBUG
 #ifdef _MSC_VER
-  _CrtDumpMemoryLeaks();
   assert(_CrtCheckMemory());
 #endif
 #endif
