@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2017 Nullsoft and Contributors
+ * Copyright (C) 1999-2021 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@
 #include "build.h"
 #include "utf.h"
 #include "util.h"
+#include "BinInterop.h"
 #include "dirreader.h"
 #include <cassert> // for assert(3)
 #include <time.h>
@@ -234,15 +235,17 @@ void CEXEBuild::del_date_time_predefines()
 TCHAR* CEXEBuild::GetMacro(const TCHAR *macroname, TCHAR**macroend /*= 0*/)
 {
   TCHAR *t = (TCHAR*)m_macros.get(), *mbeg, *mbufbeg = t;
+  size_t cbAll = m_macros.getlen();
   for (; t && *t; ++t)
   {
     mbeg = t;
+    if ((size_t)t - (size_t)mbufbeg >= cbAll) break;
     const bool foundit = !_tcsicmp(mbeg, macroname);
     t += _tcslen(t) + 1; // advance over macro name
 
     // advance over parameters
     while (*t) t += _tcslen(t) + 1;
-    t++;
+    t++; // Separator between parameters and data
 
     // advance over data
     while (*t) t += _tcslen(t) + 1;
@@ -252,15 +255,13 @@ TCHAR* CEXEBuild::GetMacro(const TCHAR *macroname, TCHAR**macroend /*= 0*/)
       if (macroend) *macroend = ++t;
       return mbeg;
     }
-    
-    if (t-mbufbeg >= m_macros.getlen()-1) break;
   }
   return 0;
 }
 
 int CEXEBuild::pp_macro(LineParser&line)
 {
-  const TCHAR*const macroname = line.gettoken_str(1);
+  const TCHAR*const macroname = line.gettoken_str(1), *tokstr;
   if (!macroname[0]) PRINTHELP()
   if (MacroExists(macroname))
   {
@@ -271,22 +272,22 @@ int CEXEBuild::pp_macro(LineParser&line)
 
   for (int pc=2; pc < line.getnumtokens(); pc++)
   {
-    if (!line.gettoken_str(pc)[0])
+    if (!(tokstr = line.gettoken_str(pc))[0])
     {
       ERROR_MSG(_T("!macro: macro parameter %d is empty, not valid!\n"), pc-1);
       return PS_ERROR;
     }
     for (int a = 2; a < pc; a++)
     {
-      if (!_tcsicmp(line.gettoken_str(pc), line.gettoken_str(a)))
+      if (!_tcsicmp(tokstr, line.gettoken_str(a)))
       {
-        ERROR_MSG(_T("!macro: macro parameter named %") NPRIs _T(" is used multiple times!\n"), line.gettoken_str(pc));
+        ERROR_MSG(_T("!macro: macro parameter named %") NPRIs _T(" is used multiple times!\n"), tokstr);
         return PS_ERROR;
       }
     }
-    m_macros.add(line.gettoken_str(pc), (int)(_tcslen(line.gettoken_str(pc))+1)*sizeof(TCHAR));
+    m_macros.add(tokstr, (int)(_tcslen(tokstr)+1)*sizeof(TCHAR));
   }
-  m_macros.add(_T(""), sizeof(_T("")));
+  m_macros.add(_T(""), sizeof(_T(""))); // Separator between parameters and data
 
   for (;;)
   {
@@ -332,7 +333,7 @@ int CEXEBuild::pp_macro(LineParser&line)
     else m_macros.add(_T(" "), sizeof(_T(" ")));
     linecnt++;
   }
-  m_macros.add(_T(""), sizeof(_T("")));
+  m_macros.add(_T(""), sizeof(_T(""))); // End of data
   return PS_OK;
 }
 
@@ -448,51 +449,26 @@ int CEXEBuild::pp_insertmacro(LineParser&line)
 int CEXEBuild::pp_tempfile(LineParser&line)
 {
   TCHAR *symbol = line.gettoken_str(1);
-  const TCHAR *fpath;
-#ifdef _WIN32
-  TCHAR buf[MAX_PATH], buf2[MAX_PATH];
-  GetTempPath(MAX_PATH, buf);
-  if (!GetTempFileName(buf, _T("nst"), 0, buf2))
+  TCHAR *tfpath = create_tempfile_path();
+  if (!tfpath)
   {
-    ERROR_MSG(_T("!tempfile: unable to create temporary file.\n"));
+    ERROR_MSG(_T("!tempfile: Unable to create temporary file!\n"));
     return PS_ERROR;
   }
-  fpath = buf2;
-#else // !_WIN32
-  char t[] = ("/tmp/makensisXXXXXX");
-  const mode_t old_umask = umask(0077);
-  int fd = mkstemp(t);
-  umask(old_umask);
-  if (fd == -1)
-  { L_tok_p_tempfile_oom:
-    ERROR_MSG(_T("!tempfile: unable to create temporary file.\n"));
-    return PS_ERROR;
-  }
-  close(fd);
-#ifdef _UNICODE
-  if (!(fpath = NSISRT_mbtowc(t))) goto L_tok_p_tempfile_oom;
-#else
-  fpath = t;
-#endif
-#endif // ~_WIN32
-
-  if (definedlist.add(symbol, fpath))
+  int symexisted = definedlist.add(symbol, tfpath);
+  free(tfpath);
+  if (symexisted)
   {
     ERROR_MSG(_T("!tempfile: \"%") NPRIs _T("\" already defined!\n"), symbol);
     return PS_ERROR;
   }
-  SCRIPT_MSG(_T("!tempfile: \"%") NPRIs _T("\"=\"%") NPRIs _T("\"\n"), symbol, fpath);
-#if !defined(_WIN32) && defined(_UNICODE)
-  NSISRT_free(fpath);
-#endif
   return PS_OK;
 }
 
 int CEXEBuild::pp_delfile(LineParser&line)
 {
-  int fatal = 1;
-  int a = 1;
-  TCHAR *fc = line.gettoken_str(a);
+  UINT fatal = true, a = 1, matchcount = 0;
+  const TCHAR *fc = line.gettoken_str(a);
   if (line.getnumtokens()==3)
   {
     if (!_tcsicmp(fc,_T("/nonfatal")))
@@ -501,7 +477,8 @@ int CEXEBuild::pp_delfile(LineParser&line)
       PRINTHELP();
   }
 
-  SCRIPT_MSG(_T("!delfile: \"%") NPRIs _T("\"\n"), line.gettoken_str(a));
+  SCRIPT_MSG(_T("!delfile: \"%") NPRIs _T("\"\n"), fc);
+  const TCHAR *fmt = _T("!delfile: \"%") NPRIs _T("\" couldn't be deleted.\n");
 
   tstring dir = get_dir_name(fc), spec = get_file_name(fc);
   tstring basedir = dir + PLATFORM_PATH_SEPARATOR_STR;
@@ -509,24 +486,32 @@ int CEXEBuild::pp_delfile(LineParser&line)
 
   boost::scoped_ptr<dir_reader> dr( new_dir_reader() );
   dr->read(dir); // BUGBUG: PATH_CONVERT?
-
-  for (dir_reader::iterator files_itr = dr->files().begin();
-       files_itr != dr->files().end();
-       files_itr++)
+  dir_reader::iterator files_itr = dr->files().begin();
+  for (; files_itr != dr->files().end(); files_itr++)
   {
     if (!dir_reader::matches(*files_itr, spec))
       continue;
 
+    ++matchcount;
     tstring file = basedir + *files_itr; // BUGBUG: PATH_CONVERT?
-
-    int result = _tunlink(file.c_str());
-    if (result == -1)
+    fc = file.c_str();
+    if (-1 == _tunlink(fc))
     {
-      ERROR_MSG(_T("!delfile: \"%") NPRIs _T("\" couldn't be deleted.\n"), file.c_str());
-      if (fatal) return PS_ERROR;
+      if (fatal)
+        return (ERROR_MSG(fmt, fc), PS_ERROR);
+      else
+        warning_fl(DW_PP_DELFILE_DELERROR, fmt, fc);
     }
     else
-      SCRIPT_MSG(_T("!delfile: deleted \"%") NPRIs _T("\"\n"), file.c_str());
+      SCRIPT_MSG(_T("!delfile: deleted \"%") NPRIs _T("\"\n"), fc);
+  }
+
+  if (!matchcount)
+  {
+    if (fatal)
+      return (ERROR_MSG(fmt, fc), PS_ERROR);
+    else
+      warning_fl(DW_PP_DELFILE_NOMATCH, fmt, fc);
   }
   return PS_OK;
 }
@@ -579,28 +564,63 @@ int CEXEBuild::pp_appendfile(LineParser&line)
   return PS_OK;
 }
 
-int CEXEBuild::pp_getdllversion(LineParser&line)
+enum { PPGVHF_VALID = 0x01, PPGVHF_NOERRORS = 0x02, PPGVHF_PACKED = 0x04, PPGVHF_TLB = 0x08 };
+int CEXEBuild::pp_getversionhelper(const TCHAR *cmdname, const TCHAR *path, const TCHAR *basesymname, DWORD high, DWORD low, DWORD flags)
 {
-  const TCHAR *cmdname = _T("!getdllversion");
-  DWORD low, high; 
-  if (!GetDLLVersion(line.gettoken_str(1), high, low))
+  TCHAR *symbuf = m_templinebuf;
+  DWORD tlb = (flags & PPGVHF_TLB), noerrors = (flags & PPGVHF_NOERRORS);
+  FILE *pF = tlb ? MSTLB_fopen(path) : FOPEN(path, ("rb"));
+  if (pF) fclose(pF);
+  bool vnum = pF && (flags & PPGVHF_VALID); // LibraryLocal users want to detect "file not found" vs "no version info"
+  if (!vnum) high = low = 0;
+  DWORD vals[] = { high >> 16, high & 0xffff, low >> 16, low & 0xffff }, count = 4;
+  if (tlb) count = 2, vals[0] = high, vals[1] = low, vals[2] = vals[3] = 0;
+
+  if (!pF)
   {
-    ERROR_MSG(_T("%") NPRIs _T(": error reading version info from \"%") NPRIs _T("\"\n"), cmdname, line.gettoken_str(1));
+    if (noerrors) return PS_OK;
+    ERROR_MSG(_T("%") NPRIs _T(": error reading version info from \"%") NPRIs _T("\"\n"), cmdname, path);
     return PS_ERROR;
   }
-  TCHAR *symbuf = m_templinebuf, numbuf[30], *basesymname = line.gettoken_str(2);
-  DWORD vals[] = { high >> 16, high & 0xffff, low >> 16, low & 0xffff };
-  SCRIPT_MSG(_T("%") NPRIs _T(": %") NPRIs _T(" (%u.%u.%u.%u)->(%") NPRIs _T("<1..4>)\n"),
-    cmdname, line.gettoken_str(1), vals[0], vals[1], vals[2], vals[3], basesymname);
-  for (UINT i = 0; i < 4; ++i)
+
+  if (flags & PPGVHF_PACKED)
   {
-    _stprintf(symbuf,_T("%") NPRIs _T("%u"), basesymname, i+1);
-    _stprintf(numbuf,_T("%lu"), vals[i]);
-    definedlist.add(symbuf, numbuf);
+    SCRIPT_MSG(_T("%") NPRIs _T(": %") NPRIs _T(" (%lu.%lu)->(%") NPRIs _T("<HIGH/LOW>)\n"),
+      cmdname, path, high, low, basesymname);
+    _stprintf(symbuf,_T("%") NPRIs _T("HIGH"), basesymname), vnum ? definedlist.set_ui32(symbuf, high) : definedlist.set(symbuf, _T(""));
+    _stprintf(symbuf,_T("%") NPRIs _T("LOW"), basesymname), vnum ? definedlist.set_ui32(symbuf, low) : definedlist.set(symbuf, _T(""));
+  }
+  else
+  {
+    SCRIPT_MSG(_T("%") NPRIs _T(": %") NPRIs _T(" (%u.%u.%u.%u)->(%") NPRIs _T("<1..%d>)\n"),
+      cmdname, path, vals[0], vals[1], vals[2], vals[3], basesymname, (int) count);
+    for (UINT i = 0; i < count; ++i)
+    {
+      _stprintf(symbuf,_T("%") NPRIs _T("%u"), basesymname, i+1);
+      vnum ? definedlist.set_ui32(symbuf, vals[i]) : definedlist.set(symbuf, _T(""));
+    }
   }
   return PS_OK;
 }
 
+int CEXEBuild::pp_getversion(int which_token, LineParser&line)
+{
+  const bool tlb = TOK_P_GETTLBVERSION == which_token;
+  const TCHAR *cmdname = tlb ? _T("!gettlbversion") : _T("!getdllversion"), *path;
+  DWORD ti = 1, flags = tlb ? PPGVHF_TLB : 0, low, high;
+  for (;; ++ti)
+  {
+    if (!_tcsicmp(line.gettoken_str(ti), _T("/noerrors")))
+      flags |= PPGVHF_NOERRORS;
+    else if (!_tcsicmp(line.gettoken_str(ti), _T("/packed")))
+      flags |= PPGVHF_PACKED;
+    else
+      break;
+  }
+  if ((tlb ? GetTLBVersion : GetDLLVersion)(path = line.gettoken_str(ti), high, low)) flags |= PPGVHF_VALID;
+  return pp_getversionhelper(cmdname, path, line.gettoken_str(ti+1), high, low, flags);
+}
+
 int CEXEBuild::pp_searchreplacestring(LineParser&line)
 {
   int ignoreCase = !_tcsicmp(line.gettoken_str(1), _T("/ignorecase"));
@@ -916,10 +936,10 @@ int CEXEBuild::pp_define(LineParser&line)
   }
   else if (!_tcsicmp(define, _T("/math")))
   {
-    int value1, value2;
+    int value1, value2, tc = line.getnumtokens(), onlyval1 = 0;
     TCHAR *mathop;
 
-    if (line.getnumtokens() != 6) PRINTHELPEX(cmdnam)
+    if (tc != 5 && tc != 6) badmathsyntax: PRINTHELPEX(cmdnam)
     define = line.gettoken_str(2);
     value1 = line.gettoken_int(3);
     mathop = line.gettoken_str(4);
@@ -938,6 +958,14 @@ int CEXEBuild::pp_define(LineParser&line)
       _stprintf(value, _T("%d"), value1 | value2);
     } else if (!_tcscmp(mathop, _T("^"))) {
       _stprintf(value, _T("%d"), value1 ^ value2);
+    } else if (!_tcscmp(mathop, _T("~"))) {
+      _stprintf(value, _T("%d"), ~ value1), ++onlyval1;
+    } else if (!_tcscmp(mathop, _T("!"))) {
+      _stprintf(value, _T("%d"), ! value1), ++onlyval1;
+    } else if (!_tcscmp(mathop, _T("&&"))) {
+      _stprintf(value, _T("%d"), value1 && value2);
+    } else if (!_tcscmp(mathop, _T("||"))) {
+      _stprintf(value, _T("%d"), value1 || value2);
     } else if (!_tcscmp(mathop, _T("<<")) || !_tcscmp(mathop, _T("<<<")) ) {
       _stprintf(value, _T("%d"), value1 << value2);
     } else if (!_tcscmp(mathop, _T(">>"))) {
@@ -951,8 +979,7 @@ int CEXEBuild::pp_define(LineParser&line)
         return PS_ERROR;
       }
       _stprintf(value, _T("%d"), value1 / value2);
-    } else if (!_tcscmp(mathop, _T("%")))
-    {
+    } else if (!_tcscmp(mathop, _T("%"))) {
       if (value2 == 0)
       {
         ERROR_MSG(_T("!define /math: division by zero! (\"%i %") NPRIs _T(" %i\")\n"),value1,mathop,value2);
@@ -961,7 +988,15 @@ int CEXEBuild::pp_define(LineParser&line)
       _stprintf(value, _T("%d"), value1 % value2);
     }
     else
-      PRINTHELPEX(cmdnam)
+      goto badmathsyntax;
+
+    if (tc + onlyval1 != 6) goto badmathsyntax;
+  }
+  else if (!_tcsicmp(define, _T("/intfmt")))
+  {
+    if (line.getnumtokens() != 5) PRINTHELPEX(cmdnam)
+    define = line.gettoken_str(2);
+    _stprintf(value = mathbuf, line.gettoken_str(3), line.gettoken_int(4));
   }
   else
   {
@@ -981,12 +1016,26 @@ int CEXEBuild::pp_define(LineParser&line)
 
 int CEXEBuild::pp_undef(LineParser&line)
 {
-  if (definedlist.del(line.gettoken_str(1)))
+  UINT noerr = false, stopswitch = false, handled = 0;
+  for (int ti = 1; ti < line.getnumtokens(); ++ti)
   {
-    ERROR_MSG(_T("!undef: \"%") NPRIs _T("\" not defined!\n"), line.gettoken_str(1));
-    return PS_ERROR; // Should this be a warning?
+    const TCHAR *name = line.gettoken_str(ti);
+    if (!stopswitch && !_tcsicmp(name, _T("/noerrors")))
+    {
+      ++noerr;
+      continue;
+    }
+    stopswitch = ++handled;
+    if (definedlist.del(name) && !noerr)
+      warning_fl(DW_PP_UNDEF_UNDEFINED, _T("!undef: \"%") NPRIs _T("\" not defined!"), name);
+    else
+      SCRIPT_MSG(_T("!undef: \"%") NPRIs _T("\"\n"), name);
+  }
+  if (!handled)
+  {
+    PRINTHELP();
+    return PS_ERROR;
   }
-  SCRIPT_MSG(_T("!undef: \"%") NPRIs _T("\"\n"), line.gettoken_str(1));
   return PS_OK;
 }
 
@@ -1001,21 +1050,24 @@ int CEXEBuild::pp_packhdr(LineParser&line)
   return bufOf ? PS_ERROR : PS_OK;
 }
 
+template<class T> void slist_append(T&list, T&item)
+{
+  T prev;
+  for (prev = list; prev && prev->next;)
+    prev = prev->next;
+  (prev ? prev->next : list) = item;
+}
+
 int CEXEBuild::pp_finalize(LineParser&line)
 {
   TCHAR* cmdstr = line.gettoken_str(1);
   int validparams = false;
-  struct postbuild_cmd *newcmd, *prevcmd;
-  newcmd = (struct postbuild_cmd*) (new BYTE[FIELD_OFFSET(struct postbuild_cmd, cmd[_tcsclen(cmdstr)+1])]);
-  newcmd->next = NULL, _tcscpy(newcmd->cmd, cmdstr);
-  newcmd->cmpop = line.gettoken_enum(2, _T("<\0>\0<>\0=\0ignore\0")), newcmd->cmpval = line.gettoken_int(3, &validparams);
+  postbuild_cmd *newcmd = postbuild_cmd::make(cmdstr, line.gettoken_enum(2, _T("<\0>\0<>\0=\0ignore\0")), line.gettoken_int(3, &validparams));
   if (line.getnumtokens() == 1+1)
-    newcmd->cmpop = 4, validparams = true; // just a command, ignore the exit code
+    newcmd->cmpop = 4, validparams = true; // Just a command, ignore the exit code
   if (newcmd->cmpop == -1 || !validparams)
     PRINTHELP();
-  for (prevcmd = postbuild_cmds; prevcmd && prevcmd->next;)
-    prevcmd = prevcmd->next;
-  if (prevcmd) prevcmd->next = newcmd; else postbuild_cmds = newcmd;
+  slist_append(postbuild_cmds, newcmd);
   SCRIPT_MSG(_T("!finalize: \"%") NPRIs _T("\"\n"), cmdstr);
   return PS_OK;
 }
@@ -1058,8 +1110,7 @@ int CEXEBuild::pp_execute(int which_token, LineParser&line)
 
   if (comp == 5)
   {
-    _stprintf(buf,_T("%d"),ret);
-    definedlist.set(define,buf);
+    definedlist.set_si32(define,ret);
   }
   else if (!check_external_exitcode(ret,comp,cmpv))
   {
